// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/chart/ContextChartSet2",
	"cloudera/Util"
], function(ContextChartSet2, Util) {

/**
 * This represents one or more service or host charts.
 */
return ContextChartSet2.extend({

	/**
	 * Generates the POST data for getting the time series.
	 */
	_generateChartsPostData: function(startDate, endDate) {

		var timeSeriesDescriptorsToFetch, i, contextType,
			context, metric, buf = [];

		if (this.newTimeSeriesDescriptors && this.newTimeSeriesDescriptors.length > 0) {
			timeSeriesDescriptorsToFetch = this.newTimeSeriesDescriptors;
		} else {
			timeSeriesDescriptorsToFetch = this.getAllEnabledTimeSeriesDescriptors();
		}

		this.addUrlParams(buf);

		buf.push("startTime=" + startDate.getTime());
		buf.push("endTime=" + endDate.getTime());

		for (i = 0; i < timeSeriesDescriptorsToFetch.length; ++i) {
			var metricIdx = "metrics[" + i + "]";
			metric = timeSeriesDescriptorsToFetch[i].metric;
			buf.push(metricIdx + ".metricId=" + metric.id);
			buf.push(metricIdx + ".derivativeType=" + metric.derivative);
			buf.push(metricIdx + ".derivativeScale=" + metric.scale);
			buf.push(metricIdx + ".units=" + metric.units);
			buf.push(metricIdx + ".contextName=" + timeSeriesDescriptorsToFetch[i].context.id);
			buf.push(metricIdx + ".contextType=" + timeSeriesDescriptorsToFetch[i].context.type);
		}

		if (this.newTimeSeriesDescriptors && this.newTimeSeriesDescriptors.length) {
			this.newTimeSeriesDescriptors = [];
		}
		return buf.join("&");
	},

	/**
	 * Generates the POST data for getting the events.
	 */
	_generateEventsPostData: function(startDate, endDate) {
		var timeSeriesDescriptorsToFetch, i, contextType,
		context, metric, buf = [];

		if (this.newTimeSeriesDescriptors && this.newTimeSeriesDescriptors.length > 0) {
			timeSeriesDescriptorsToFetch = this.newTimeSeriesDescriptors;
		} else {
			timeSeriesDescriptorsToFetch = this.getAllEnabledTimeSeriesDescriptors();
		}

		this.addUrlParams(buf);

		buf.push("timeUpper=" + endDate.getTime());
		buf.push("timeLower=" + startDate.getTime());

		for (i = 0; i < timeSeriesDescriptorsToFetch.length; ++i) {
			context = timeSeriesDescriptorsToFetch[i].context;
		}

		if (context) {
			buf.push(encodeURIComponent(context.type.toLowerCase()) + "=" +
					encodeURIComponent(context.id));
		}

		buf.push("summarise=on");
		var duration = this.delta(endDate, startDate);
		// by default, we want to see at most 10 events per category for each fetch.
		var summariseSize = duration / 10;
		if (summariseSize > 0) {
			buf.push("summariseSize=" + summariseSize);
		}
		buf.push("summariseGroupBy=CATEGORY");
		buf.push("summariseBy=SEVERITY");

		return buf.join("&");
	},

	addUrlParams: function (buf) {
		if (this.options.urlParams) {
			$.each(this.options.urlParams, function(k, v) {
				buf.push(encodeURIComponent(k) + "=" + encodeURIComponent(v));
			});
		}
	},

	initializeData: function() {
		this._super.apply(this, arguments);
		/*
		this.options.timeSeriesDescriptors =
		[ {
			"metric" : {
				"id" : 10001,
				"derivative" : false,
				"context" : "ROLE",
				"desc" : "Current number of blocks",
				"title" : "Num blocks",
				"name" : "dfs_blocks",
				"selector" : "AVERAGE"
			},
			"chart" : {
				"id" : "chart10001",
				"title" : "DFS Blocks",
				"discrete" : false
			},
			"context" : {
				"id" : "roleName",
				"type" : "ROLE"
			}
		}	]
		*/
		var i, len = this.options.timeSeriesDescriptors.length;
		for (i = 0; i < len; i += 1) {
			var timeSeriesDescriptor = this.options.timeSeriesDescriptors[i];
			this.addTimeSeries(timeSeriesDescriptor);
		}
		// console.log("ServiceChartSet");
		// console.log(this.getConfig());
	},

	getSeriesName: function(timeSeriesDescriptor) {
		return timeSeriesDescriptor.metric.title;
	}
});

});
