// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
/**
 * This is a port of Humanize.java for JavaScript.
 * We need this to render JSON data in the browser.
 */
define([], function() {

	var Humanize = {
		_timezoneDelta: 0,

		_timezoneDisplayName: "",

		humanizeBytes: function(bytes) {

			var UNITS = [ "B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB" /* 2 ^ 60 */ ];
			if (bytes < 0) {
				return "-" + Humanize.humanizeBytes(-1 * bytes);
			}
			if (bytes < 1024) {
				return bytes + " B";
			}
			var index = Math.floor(Math.log(bytes) / Math.log(1024));
			index = Math.min(UNITS.length - 1, index);
			return (bytes / Math.pow(1024, index)).toFixed(1) + " " + UNITS[index];
		},

		humanizeSeconds: function(seconds) {
			return Humanize.humanizeMilliseconds(seconds * 1000);
		},

		/** Convert milliseconds into human-friendly string. */
		humanizeMilliseconds: function(ms) {
			var val = ms / 1000.0;
			if (val < 60) {
				return val.toFixed(2) + "s";
			}
			val = val / 60.0;
			if (val < 60) {
				return val.toFixed(1) + "m";
			}
			val = val / 60.0;
			if (val < 24) {
				return val.toFixed(1) + "h";
			}
			val = val / 24.0;
			return val.toFixed(1) + "d";
		},

		humanizeDateTimeMedium: function(dateObject) {
			return this._humanizeTimeWithFormat(dateObject, "LLL");
		},

		humanizeDateLong: function(dateObject) {
			return this._humanizeTimeWithFormat(dateObject, "LL");
		},

		humanizeDateTimeLong: function(dateObject) {
			return this._humanizeTimeWithFormat(dateObject, "LLLL");
		},

		humanizeTimeShort: function(dateObject) {
			return this._humanizeTimeWithFormat(dateObject, "LT");
		},

		humanizeTimeShortAndMS: function(dateObject) {
			// This is a special mode that we added locally to moment.
			// This represents showing only the time portion,
			// and add milliseconds after the second.
			return this._humanizeTimeWithFormat(dateObject, "TT");
		},


		humanizeTimeLong: function(dateObject) {
			return this._humanizeTimeWithFormat(dateObject, "H:mm:ss.SSS ZZ");
		},

		_humanizeTimeWithFormat: function(dateObject, format) {
			return moment(dateObject.getTime() + this.getTimezoneDelta()).format(format);
		},

		/**
		 * @param delta (in milliseconds).
		 */
		setTimezoneDelta: function(delta) {
			window._timezoneDelta = delta;
		},

		getTimezoneDelta: function() {
			return window._timezoneDelta;
		},

		setTimezoneDisplayName: function(displayName) {
			if (displayName) {
				window._timezoneDisplayName = displayName;
			}
		},

		getTimezoneDisplayName: function() {
			return window._timezoneDisplayName;
		},

		fromServerDate: function(serverDate) {
			return new Date(serverDate.getTime() - this.getTimezoneDelta());
		},

		toServerDate: function(localDate) {
			return new Date(localDate.getTime() + this.getTimezoneDelta());
		}
	};

	return Humanize;

});
