// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/headlamp/hdfs/include/QueryTerm"
], function(QueryTerm) {
	/**
	 * Manages file search filtering.
	 *
	 * @param options {
	 *     searchUrl: "",
	 *     queryTermsJson: "",
	 *     rulesBuilder: {
	 *         ....
	 *     }
	 * }
	 */
	return function(options) {

		var getFilterContainer = function() {
			return $("#fileSearchFilterContainer");
		};

		var isFormValid = function() {
			return getFilterContainer().closest("form").valid();
		};

		var onKeypressedInSearchFilter = function(evt) {
			var code = evt.keyCode || evt.which;
			// if user pressed enter, triggers the search button.
			if ($.ui.keyCode.ENTER === code) {
				viewModel.search();
			}
		};

		var viewModel = {
			rulesBuilder: options.rulesBuilder,

			selectedSearchOption: ko.observable("basic"),

			search : function() {
				if (isFormValid()) {
					var queryTerms = [], queryTermsJson, rules = this.rulesBuilder.rules();
					// convert rules to queryTerms.
					$.each(rules, function(index, rule) {
						var queryTerm = new QueryTerm(rule);
						if (queryTerm.fileSearchType !== undefined) {
							queryTerms.push(queryTerm);
						}
					});
					var searchMode = false;
					if (rules.length > 0 && queryTerms.length > 0) {
						queryTermsJson = JSON.stringify({ terms: queryTerms });
						searchMode = true;
					} else {
						// go back to default of the page.
						queryTermsJson = options.queryTermsJson;
					}
					jQuery.publish("fileSearchFilterChanged", [options.searchUrl,  queryTermsJson, searchMode]);
				}
			},

			convertQueryTermToRule: function(term) {
				var ruleType = "string";
				var ruleValue = "";
				var ruleComparator = "";
				// deduce the ruleType based on the field information.
				if (term.startOfRange !== undefined || term.endOfRange !== undefined) {
					ruleType = "numeric";
					if (term.startOfRange && term.endOfRange) {
						// We don't support this today
						// if startOfRange is different from
						// the endOfRange.
						ruleValue = term.startOfRange;
						ruleComparator = "EQ";
					} else if (term.startOfRange) {
						ruleValue = term.startOfRange;
						if (term.startInclusive) {
							ruleComparator = "GTE";
						} else {
							ruleComparator = "GT";
						}
					} else if (term.endOfRange) {
						ruleValue = term.endOfRange;
						if (term.endInclusive) {
							ruleComparator = "LTE";
						} else {
							ruleComparator = "LT";
						}
					}
				} else if (term.queryText !== undefined) {
					ruleValue = term.queryText;
					ruleCompartor = "EQ";
				}
				var ruleName = String(term.fileSearchType);
				// create a new rule.
				return this.rulesBuilder.newRule(ruleName, ruleComparator, ruleValue);
			},

			/**
			 * Populate rules from a JSON structure.
			 */
			populateRules : function(queryTerms) {
				var self = this;
				$.each(queryTerms.terms, function(i, term) {
					var rule = self.convertQueryTermToRule(term);
					self.rulesBuilder.rules.push(rule);
				});
			}
		};

		viewModel.filterButtonSelected = ko.dependentObservable(function() {
			return (viewModel.rulesBuilder.rules().length > 0);
		}, viewModel);

		getFilterContainer()
			.keypress(onKeypressedInSearchFilter)
			.closest("form").validate();

		$("#fileSearchAdvanced ul a").click(function(event) {
			var $target = $(event.target);
			if ($target.length > 0) {
				var id = $target.attr("data-query-id");
				var searchUrl = $target.find(".searchUrl").text();
				var queryTermsJson = $target.find(".queryTermsJson").text();

				viewModel.selectedSearchOption(id);

				// repopulate the search filter.
				viewModel.rulesBuilder.rules.removeAll();

				var queryTerms = $.parseJSON(queryTermsJson);
				if (queryTerms && $.isArray(queryTerms.terms)) {
					viewModel.populateRules(queryTerms);
					// Custom... has id "advanced",
					// This need further user input, cannot search
					// automatically.
					if (id !== "advanced") {
						// automatically trigger search.
						viewModel.search();
					}
				} else if (id !== "basic") {
					jQuery.publish("fileSearchFilterChanged", [searchUrl, queryTermsJson, true]);
				}
			}
			$target = null;
		});

		viewModel.basicSearch = function() {
				var basicSearchJson = $("#fileSearchBasicJson").text();
				var searchUrl = options.searchUrl;
				var queryTermsJson = options.queryTermsJson;
				var searchMode = false;

				var queryTerms = $.parseJSON(basicSearchJson);
				if (queryTerms && $.isArray(queryTerms.terms)) {
					var searchTerm = $.trim($("#fileSearchBasic").val());
					if (searchTerm) {
						queryTerms.terms[0].queryText = "*" + searchTerm + "*";
						queryTermsJson = JSON.stringify(queryTerms);
						searchMode = true;
					}
				}
				jQuery.publish("fileSearchFilterChanged", [searchUrl, queryTermsJson, searchMode]);
		};
		return viewModel;
	};
});
